// import 'dart:async';
import 'package:crypto/crypto.dart';
import 'package:flutter/services.dart';
import 'dart:async';
import './types.dart';
import 'dart:convert';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:socket_io_client/socket_io_client.dart';
import 'package:http/http.dart' as http;
import './webview.dart';
import 'package:device_info/device_info.dart';
import './const.dart';
export './types.dart';

// String urlDev = "http://192.168.66.182:12003";

class Simpay {
  static const MethodChannel _channel = MethodChannel('plugin_simpay');
  static final DeviceInfoPlugin deviceInfoPlugin = DeviceInfoPlugin();
  String commerceCode;
  String apiKey;
  String secretKey;
  String prod = URL_PRODUCTION;
  String dev = URL_SANDBOX;
  String idenifierDevice = INIT_IDENTIFIER;
  String _title = TITLE_WEBVIEW;
  bool production;
  Color _colorAppBar = Colors.white;
  Color _colorText = Colors.black;
  dynamic ws;
  dynamic contextCache;

  Simpay({
    Key? key,
    required this.commerceCode,
    required this.apiKey,
    required this.secretKey,
    this.production = false,
  }) {
    openConnect();
  }

  StreamController<ResPayment> _res = StreamController<ResPayment>.broadcast();

  Socket getSocket(String url) {
    print("GET_SOCKET");
    return io(
        url,
        OptionBuilder()
            .setTransports(['websocket'])
            .disableAutoConnect()
            .setPath("/ws")
            .build());
  }

  String signSimple() {
    var hmacSha256 = Hmac(sha256, utf8.encode(secretKey));
    var digest = hmacSha256.convert(utf8.encode(commerceCode + apiKey));
    return digest.toString();
  }

  String sign(order, amount) {
    var hmacSha256 = Hmac(sha256, utf8.encode(secretKey));
    var digest = hmacSha256.convert(utf8
        .encode(commerceCode + apiKey + order.toString() + amount.toString()));
    return digest.toString();
  }

  String getHeaderSignSimple() {
    return commerceCode + SIGNAL_FIRM + signSimple();
  }

  Uri getUrl([String path = '/']) {
    return Uri.parse((production ? prod : dev) + path);
  }

  Map<String, String>? getHeader(String sign) {
    return {
      "sgn": commerceCode + SIGNAL_FIRM + sign,
      "Content-Type": "application/json"
    };
  }

  Future<ListPayments> getMethodsPayments() async {
    var url = getUrl(API_GET_METHODS);
    var header = getHeader(signSimple());
    var res = await http.get(url, headers: header);

    if (res.statusCode == 200) {
      ListPayments payments = ListPayments.fromJson(json.decode(res.body));
      return payments;
    } else {
      print("Error: ${res.body}");
      return new ListPayments(list: []);
    }
  }

  Future<ResponsePayment> createPay(DataPayments data) async {
    idenifierDevice = await initPlatformState();
    var url = getUrl(POST_PAYMENT);
    var header = getHeader(sign(data.order, data.amount));
    data.commerceCode = commerceCode;
    data.mobile = true;
    data.deviceId = idenifierDevice;
    var res = await http.post(url, body: data.JSON(), headers: header);

    if (res.statusCode == 200) {
      return ResponsePayment.fromJson(json.decode(res.body));
    } else {
      throw Exception(ERR_CREATE_PAYMENT);
    }
  }

  void getPay(BuildContext context, ResponsePayment data) {
    contextCache = context;
    Navigator.push(
      context,
      MaterialPageRoute(
          builder: (context) => PayWebView(
                url: data.url.replaceAll('http://', 'https://'),
                title: _title,
                colorAppBar: _colorAppBar,
                colorText: _colorText,
              )),
    );
  }

  void openConnect() {
    String url = this.production ? this.prod : this.dev;
    Socket socket = io(
        url,
        OptionBuilder()
            .setTransports(['websocket'])
            .disableAutoConnect()
            .setPath("/ws")
            .build());

    ws = socket;
    socket.connect();
    socket.onConnect((_) => print(WS_CONNECT));
    socket.onDisconnect((_) => print(WS_DISCONNECT));
    socket.on(WS_EVENT, (res) {
      if (idenifierDevice.compareTo(res['identifier']) == 0) {
        StatusPayment s = getStatusPayment(res['data']['status']);
        ResPayment data = ResPayment(s, res['data']);
        _res.add(data);
        Future.delayed(Duration(milliseconds: 500), () {
          Navigator.of(contextCache, rootNavigator: true).pop();
        });
      }
    });
  }

  void closeConnection() {
    ws.disconnect();
    ws.destroy();
  }

  StatusPayment getStatusPayment(String stats) {
    return StatusPayment.values
        .firstWhere((e) => e.toString() == 'StatusPayment.' + stats);
  }

  StreamController<ResPayment> get success => _res;

  Future<String> initPlatformState() async {
    try {
      if (Platform.isAndroid) {
        AndroidDeviceInfo? device = await deviceInfoPlugin.androidInfo;
        return "${device.androidId}-${commerceCode}";
      } else if (Platform.isIOS) {
        IosDeviceInfo? device = await deviceInfoPlugin.iosInfo;
        return "${device.identifierForVendor}-${commerceCode}";
      } else {
        return ERR_NOT_DEVICE;
      }
    } catch (e) {
      return ERR_NOT_DEVICE;
    }
  }

  void configViewPay({String? title, Color? colorAppBar, Color? colorText}) {
    if (title != null) _title = title;
    if (colorAppBar != null) _colorAppBar = colorAppBar;
    if (colorText != null) _colorText = colorText;
  }

  static String get version {
    return "0.0.5";
  }

  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }
}
