import 'dart:convert';

class ResponsePayment {
  final String url;
  final String order;
  final String session;
  final dynamic amount;
  final bool success;
  ResponsePayment(
      {required this.success, required this.order, required this.url, required this.session, this.amount});

  factory ResponsePayment.fromJson(Map<String, dynamic> json) {
    return ResponsePayment(
        success: json['success'],
        url: json['data']['url'],
        order: json['data']['order'],
        session: json['data']['session'],
        amount: json['data']['amount']);
  }

  String JSON() {
    return json.encode({
      "success": this.success,
      "url": this.url,
      "order": this.order,
      "amount": this.amount,
      "session": this.session,
    });
  }
}

class PaymentMethods {
  final int code;
  final String name;
  PaymentMethods({required this.code, required this.name});

  factory PaymentMethods.fromJson(Map<String, dynamic> json) {
    return PaymentMethods(
      code: json['data']['code'],
      name: json['data']['name'],
    );
  }
}

class ListPayments {
  final List<PaymentMethods> list;
  ListPayments({required this.list});

  factory ListPayments.fromJson(Map<String, dynamic> json) {
    return ListPayments(
      list: List<Map<String, dynamic>>.from(json['data']).map((e) {
        return PaymentMethods(code: e['code'], name: e['name']);
      }).toList(),
    );
  }
}

class DataPayments {
  String? commerceCode;
  bool? mobile;
  String? deviceId;
  int amount;
  String order;
  String? subject;
  int? method;
  List<Map<String, dynamic>>? items;
  DataPayments(
      {this.commerceCode,
      this.deviceId,
      this.mobile,
      required this.amount,
      required this.order,
      this.subject,
      this.method,
      this.items});

  String JSON() {
    return json.encode({
      "commerceCode": this.commerceCode,
      "deviceId": this.deviceId,
      "mobile": this.mobile,
      "amount": this.amount,
      "order": this.order,
      "subject": this.subject,
      "method": this.method,
      "items": this.items
    });
  }
}

enum StatusPayment { AUTHORIZED, CANCEL, FAILED }

class DRPayment {
  final StatusPayment status;
  final String identifier;
  final DRIPayment data;
  DRPayment(
      {required this.status, required this.identifier, required this.data});

  factory DRPayment.fromJson(Map<String, dynamic> json) {
    StatusPayment s = StatusPayment.values
        .firstWhere((e) => e.toString().compareTo(json['data']['status']) == 0);
    return DRPayment(
      status: s,
      identifier: json['identifier'],
      data: DRIPayment.fromJson(s, json['data']['details']),
    );
  }
}

class DRIPayment {
  final StatusPayment status;
  final String order;
  final int amound;
  final Map<String, dynamic> details;
  DRIPayment(
      {required this.status,
      required this.order,
      required this.amound,
      required this.details});

  factory DRIPayment.fromJson(StatusPayment status, Map<String, dynamic> json) {
    return DRIPayment(
      status: status,
      order: json['order'],
      amound: json['amound'],
      details: json['details'],
    );
  }
}

class ResPayment {
  StatusPayment status;
  Map<String, dynamic> data;

  ResPayment(this.status, this.data);
}
